#**********************************************************************************************************#
# Méthode de Monte-Carlo pour le calcul de la valeur moyenne et de l'incertitude-type de la concentration  #
# molaire Cf de la solution fille Si (i compris entre 1 et 5)                                              #
# Programme publié par le GRIESP en 2019 - Modifications mineures apportées par Marie-Anne DEJOAN          #
#**********************************************************************************************************#

from scipy import *
import numpy as np
from matplotlib import pyplot as plt

# Renvoie une valeur aléatoire de la variable L[0] d'incertitude-type L[1]
def Alea(L):
    tirage=np.random.normal()   # Tirage entre -infini et +infini (loi normale)
    return L[0]+L[1]*tirage

# Entrées
m=[584.e-3,1.e-3] # [valeur,incertitude-type] de chaque entrée
M=[58.44,0.01]
Vo=[0.50000,0.00025]

# Incertitudes sur la verrerie utilisée
u_fiole=0.06e-3      # Fiole jaugée à 50 mL
u_pipette5=0.03e-3   # Pipette jaugée à 5 mL
u_pipette10=0.04e-3  # Pipette jaugée à 10 mL
u_pipette20=0.06e-3  # Pipette jaugée à 20 mL

#----------------------Partie du script à compléter par les élèves-------------------------
numero=1   # Solution fille i (i compris entre 1 et 5)

# Prélevement du volume Vm de solution mère :
coef_5=0   # nombre de prélevements réalisés avec la pipette jaugée à 5mL (0, 1 ou 2)
coef_10=1  # avec la pipette jaugée à 10mL (0, 1 ou 2)
coef_20=0  # avec la pipette jaugée à 20mL (0, 1 ou 2)

#Volume de fiole utilisée (inscrire 1 pour la fiole utilisée, 0 pour l'autre)
Nfiole50=1
Nfiole100=0

# Incertitude composée sur le volume prélevé de solution mère à l'aide des pipettes jaugées
u_pipettes=sqrt(coef_5*(u_pipette5**2)+coef_10*(u_pipette10**2)+coef_20*(u_pipette20**2))

Vm=[coef_5*5e-3+coef_10*10e-3+coef_20*20e-3,u_pipettes]   # Volume prélevé de la solution mère et incertitude composée
Vf=[Nfiole50*50e-3+Nfiole100*100e-3,u_fiole]              # Volume de la solution fille
#------------------------------------------------------------------------------------------

# Calcul de Cm
Cm=m[0]/(M[0]*Vo[0]) # Cm: concentration de la solution mère

# Calcul de Cf
Cf=Cm*Vm[0]/Vf[0] # Cf: concentration de la solution fille (Si)

# Méthode de Monte Carlo pour trouver l'incertitude sur Cf sans formule de propagation des incertitudes
LCf=[] # Initialisation de la liste LCf qui contiendra les valeurs successives de Cf calculées par itération
iteration=100000

for i in range(iteration):
    AleaCm=Alea(m)/(Alea(M)*Alea(Vo))
    AleaCf=AleaCm*Alea(Vm)/Alea(Vf)
    LCf.append(AleaCf) # Ajout de la dernière valeur calculée à la liste LCf

MoyCf=mean(LCf)        # Calcul de la valeur moyenne de Cf (à partir des éléments qui composent la liste LCf)
uCf=std(LCf, ddof = 1) # Ecart-type expérimental = incertitude-type de la grandeur Cf

# Affichage
print('Solution fille n°',numero)
print('Calcul de Cf :',Cf,'mol/L')
print('Moyenne des Cf :',MoyCf,'mol/L')
print('u(Cf) :',uCf,'mol/L')

